/**
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package contents;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import de.trantor.mail.Message;
import de.trantor.mail.MimeDecoder;
import util.Datas;
import mmc.StartMidlet;
import threads.InboxThread;
import java.util.Vector;
import media.*;

/**
 * Screen for reading a mail
 * @author Gabriele Bianchi
 *
 */
public class ReadScreen extends Form implements ItemCommandListener{


    
		
    public static Command BACK = new Command("Back", Command.BACK, 0);
    public static Command REPLY = new Command("Reply", Command.SCREEN, 1);
    public static Command DELETE = new Command("Delete", Command.SCREEN, 2);
    public static Command FORWARD = new Command("Forward", Command.SCREEN, 3);
    public static Command ATTACH = new Command("Open file", Command.SCREEN, 4);
	
    public ChoiceGroup attaches = new ChoiceGroup("Attachments:", Choice.EXCLUSIVE);

    private InboxThread inboxThread;
    private MimeDecoder mime;
    private PlayerManager player;
    private String alert = null;
    private Vector infopool;
    public Message message;
    public StartMidlet midlet;
    public int messageId;

    public ReadScreen(StartMidlet _midlet, int index) {
        super("Message");
        midlet = _midlet;
		    messageId = index;
        inboxThread = _midlet.inboxThread;
        infopool = new Vector();
       
        try {
        	message = inboxThread.getMessage(index);
        }catch (Exception e) {midlet.returnToInboxError(e.getMessage());}
        
        this.append(new StringItem("Subject:", message.getHeaderValue("subject", "No Subject")));
      	this.append(new StringItem("From:", message.getHeaderValue("from", "No Sender")));
      
		try
		{
			mime = new MimeDecoder(message);
		}
		catch (Exception e1) { midlet.returnToInboxError("Attachments are too big to be opened!"); }
		
        int parts = mime.getPartCount();
        System.out.println("Parts:" + parts);
        if (parts == 0) {
        	int max = mime.getBodyLineCount();
		      	for (int i=0; i < max; i++)
		      		//this.append(mime.getBodyLine(i));
					appendText(mime.getBodyLine(i));
        }
        else {
	        for (int j = 0; j < parts; j++) {
	      		MimeDecoder temp = mime.getPart(j);
	      		if (temp.getType().equals("text/plain") || temp.getType().equals("text/html") || temp.getType().equals("text/richtext")) { 
			      	int max = temp.getBodyLineCount();
			      	for (int i=0; i < max; i++)
			      		this.append(temp.getBodyLine(i));
			      }
			      else if (temp.getType().startsWith("image") || temp.getType().startsWith("audio") || temp.getType().startsWith("video")) {
			      	
			      	attaches.append(temp.getName(), GenericContents.displayImage("attach"));
			      	//infopool.addElement(temp);
					infopool.addElement(new Integer(j));
			      }
			      else if(temp.getType().equals("application/pdf"))
			      	alert = "There is a PDF attachment, I can't open it";
			      else if(temp.getType().equals("application/msword"))
			      	alert = "There is a WORD attachment, I can't open it";
			      else if(temp.getType().equals("application/zip"))
			      	alert = "There is a ZIP attachment, I can't open it";
			      else 
			      	alert = "There are some unknown attachments!";
	      	}	
      	}
      	if (attaches.size() > 0) {
      		attaches.addCommand(ATTACH);
      		attaches.setItemCommandListener(this);
      		this.append(attaches);
      	}
      	if (alert != null)
      		this.append(alert);
      	
     // }
      
      
      	addCommand(BACK);
      	addCommand(REPLY);
      	addCommand(DELETE);
      	addCommand(FORWARD);
      	setCommandListener(midlet);
        	
       
        
        
    }
    
  /**
	 * @param Command
	 * @param Item
	*/
	public void commandAction(Command cmd, Item i) {
		if (cmd == ATTACH)
		{
			if (infopool.size() == 0)
			{
				midlet.returnToInboxError("No attachment found!");
				return;
			}

			Integer j = (Integer)infopool.elementAt(((ChoiceGroup)i).getSelectedIndex());
			try
			{
				player = new PlayerManager(midlet, mime.getPart(j.intValue()), this);
			}
			catch (java.lang.OutOfMemoryError e1) { midlet.returnToInboxError("Attachments are too large to be opened!"); }


			(new Thread(player)).start();
		}
		else //link activation
		{
			MediaManager.activateLink(((StringItem)i).getLabel(), midlet);
		}

	}

	/**
	 * Find links in the body
     */
	private void appendText(String body)
	{
		int i = -1;
		if ((i = body.indexOf("http://")) != -1 || (i = body.indexOf("https://")) != -1)
		{
			int end = body.indexOf(" ", i);
			if (end == -1)
				end = body.length();
			this.append(body.substring(0, i));
			StringItem link = new StringItem(body.substring(i, end), "");
			this.append(link);
			this.append(body.substring(end, body.length()));
			link.addCommand(new Command("Activate link", Command.ITEM, 5));
			link.setItemCommandListener(this);

		}
		else
			this.append(body);
	}
    

    
    
}
