/**
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package mmc;

import java.io.IOException;

import java.util.Vector;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import util.Datas;
import contents.*;
import threads.*;
import de.trantor.mail.*;
import media.MediaManager;

/**
 * Application midlet
 * @author Gabriele Bianchi
 *
 */
public class StartMidlet extends MIDlet implements CommandListener {

    public Display display;

	private StartScreen startScreen;
    private SetupScreen setupScreen;
    private MainScreen mainScreen;
    private ComposeScreen composeScreen;
	private ReadScreen readScreen;

    

    public InboxThread inboxThread;
   


    private Vector msgNumbers = new Vector();

    public StartMidlet() {
      
            Datas.load();
						
     
    }
/**
 * 
 * @param cmd
 * @param dsp
 */
    public void commandAction(Command cmd, Displayable dsp) {
        
        try {
        	if (cmd == StartScreen.EXIT) {
				if (inboxThread != null)
					inboxThread.close();
        		notifyDestroyed();
        	}
			else if (cmd == SetupScreen.BACK)
			{
				display.setCurrent(getGuiStartScreen());
			}
        	else if (cmd == SetupScreen.OK) {
								
				Datas.address  = setupScreen.getAddress();
                Datas.localhost = setupScreen.getHostname();
                Datas.pop3Host = setupScreen.getPop3Host();
                Datas.pop3User = setupScreen.getPop3User();
                Datas.password = setupScreen.getPop3Pass();
                Datas.smtpHost = setupScreen.getSmtpHost();
                Datas.smtpAuth = setupScreen.getSmtpAuth();
                Datas.imap = setupScreen.getImap();
                Datas.ssl = setupScreen.getPop3Ssl();
                Datas.port = setupScreen.getPop3Port();              
                try {
                    Datas.saveSetupData();
                }
                catch (Exception e) {
                    e.printStackTrace();
					genericSaveError(e.getMessage());

                }
				if (GenericContents.saved.getImage() == null)
					GenericContents.saved.setImage(GenericContents.displayImage("save"));
				display.setCurrent(GenericContents.saved, getGuiStartScreen());
                
            }
            else if (cmd == MainScreen.BACK) {
            	
            
            	display.setCurrent(getGuiStartScreen());
            }
            else if (cmd == MainScreen.WRITE) {
            	display.setCurrent(getGuiComposeScreen());
            }
            else if (cmd == MainScreen.REFRESH) {
            	Datas.update = true;
            	display.setCurrent(getGuiInboxScreen());
            }
            else if (cmd == MainScreen.READ) {
            	try {
            		if (GenericContents.alertRead.getImage() == null)
            			GenericContents.alertRead.setImage(GenericContents.displayImage("wait"));
            		display.setCurrent(GenericContents.alertRead, getGuiReadScreen(mainScreen.mails.getSelectedIndex()));
            	}catch (java.lang.OutOfMemoryError e1) { returnToInboxError("Attachments are too large to be opened!"); }
            }
            else if (cmd == ReadScreen.BACK) {
            	display.setCurrent(getGuiInboxScreen());
            }
			else if (cmd == ReadScreen.DELETE) {
				try {
					inboxThread.removeMessage(readScreen.messageId);
				}catch (MailException ex) {returnToMessageError(ex.getMessage());}
				catch (IOException ex2) {returnToInboxError(ex2.getMessage());}
				Datas.update = true;
				display.setCurrent(getGuiInboxScreen());
			}
			else if (cmd == ReadScreen.FORWARD) {
				display.setCurrent(getGuiComposeScreenForw(readScreen.message));

			}
			else if (cmd == ReadScreen.REPLY)
			{
				display.setCurrent(getGuiComposeScreenReply(readScreen.message));
			}

          
        }
        catch (Exception error) {
        		inboxThread.close();
            Alert alert = new Alert("Error");
            alert.setString(error.getClass().getName() + ": " + error.getMessage());
            error.printStackTrace();
            display.setCurrent(alert, getGuiParamScreen());
        }
    }
	/**
	 * startApplication
	 * @throws MIDletStateChangeException
	 */
    protected void startApp() throws MIDletStateChangeException {
    	//TODO: inserire il controllo se  una ripartenza..
		if (display == null)
			display = Display.getDisplay(this);
        display.setCurrent(getGuiIntroScreen(), getGuiStartScreen());
    }
    /**
     * destroyApplication
     * @param unconditional
     */
    public void destroyApp(boolean unconditional) {
        try {
            inboxThread.close();
        }
        catch (Exception error) {
        }
    }
    /**
     * pauseApplication
     *
     */
    protected void pauseApp() {
        try {
        }
        catch (Exception error) {
        }
    }

   /**
	 * Show the intro logo and author info 
	 * @return Alert
	 */
	 public Alert getGuiIntroScreen() {
		Image logo;
		try {
		 logo = Image.createImage("/mmc_logo.png");
		}catch(java.io.IOException e){
		 logo = null;
		}	 
	 	Alert intro = new Alert("Mobile Mail Checker", " Created by Gabriele Bianchi", logo, AlertType.INFO);
	 try
    		{
      		//Sound
      		    MediaManager.playSound("/intro.mp3", "audio/mpeg");
    		}
    		catch (Exception e) { System.out.print(e.getMessage());}
		return intro;
	 }
	 /**
	 * Show the settings screen 
	 * @return Form
	 */
	 public Form getGuiParamScreen() {
	 	setupScreen = new SetupScreen(this);
	 	return setupScreen;
	 	
	 }
	 /**
	 * Show the start screen 
	 * @return Form
	 */
	 public Form getGuiStartScreen()
	 {

		 if (startScreen == null)
			 startScreen = new StartScreen(this);
		 return startScreen;

	 }

	 /**
	 * Show the inbox panel
	 * @return Form
	 */
	 public Form getGuiInboxScreen() {
		if (mainScreen == null || Datas.update) {
			
	 		mainScreen = new MainScreen(this);
			Datas.update = false;
		}
	 	return mainScreen;
	 	
	 }
	 /**
	 * Show the compose panel
	 * @return Form
	 */
	 public Form getGuiComposeScreen() {
	
	 	composeScreen = new ComposeScreen(this);
	 	return composeScreen;
	 	
	 }
	 /**
	  * Show the compose panel for forward
	  * @param mesage
	  * @return Form
	  */
	 public Form getGuiComposeScreenForw(Message message)
	 {

		 composeScreen = new ComposeScreen(this, message, ComposeScreen.title_forw, false);
		 return composeScreen;

	 }
	 /**
	   * Show the compose panel for reply
	   * @param mesage
	   * @return Form
	   */
	public Form getGuiComposeScreenReply(Message message)
	 {

		 composeScreen = new ComposeScreen(this, message, ComposeScreen.title_reply, true);
		 return composeScreen;

	 }
	 /**
	 * Show the message panel
	 * @return Form
	 */
	 public Form getGuiReadScreen(int index) {
		 int realIndex = mainScreen.mails.size() - 1 - index;//attenzione
		 readScreen = new ReadScreen(this, realIndex);
	 	 return readScreen;
	 }
	 /**
	  * Error notification
	  * @param exc
	  */
	 public void notifyBadConnection(String exc) {
	 	display.setCurrent(new Alert("Error", exc, null, AlertType.INFO), getGuiStartScreen());
	 }
	 /**
	  * Error notification
	  * @param exc
	  */
	 public void genericErrorConnection(String exc) {
	 	display.setCurrent(new Alert("Error", exc, null, AlertType.INFO), getGuiStartScreen());
	 }
	 /**
	  * Error notification
	  * @param exc
	  */
	public void genericSaveError(String exc)
	{
		display.setCurrent(new Alert("Error", exc, null, AlertType.INFO), getGuiParamScreen());
	}
	/**
	  * Error notification
	  * @param exc
	  */
	 public void returnToInboxError(String exc) {
	 	display.setCurrent(new Alert("Error", exc, null, AlertType.INFO), getGuiInboxScreen());
	 }
	 /**
	  * Error notification
	  * @param exc
	  */
	 public void returnToMessageError(String exc) {
		if (readScreen != null)
	 		display.setCurrent(new Alert("Error", exc, null, AlertType.INFO), readScreen);
		else
			returnToInboxError(exc);
	 }
	
	
}
