/**
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package threads;

import java.lang.Thread;
import contents.ComposeScreen;
import de.trantor.mail.SmtpClient;
import de.trantor.mail.Message;
import de.trantor.mail.Envelope;
import mmc.MMConnection;
import util.Datas;
/**
 * Thread for the smtp connections
 * @author Gabriele Bianchi
 *
 */
public class SmtpThread extends Thread {

	private String localhost;
	private ComposeScreen compScreen;
	private MMConnection conn;
	private boolean exit;
	public SmtpClient smtpClient;
	private Message msg_toSend = null;

	/**
	 * Constructor
	 * @param local
	 * @param _compScreen
	 */
	public SmtpThread(String local, ComposeScreen _compScreen) {
		localhost = local;
		compScreen = _compScreen;
		conn = new MMConnection();
	}
	/**
	 * Run the thread
	 */
	public void run() {
		connect();
		exit = false;
		while (!exit)
		{
			if (msg_toSend == null)
			{
				try
				{
					wait(1000);  // waits for something to send. 
				}
				catch (InterruptedException e)
				{
					// when does it happen?
					System.out.println("[INTERRUPTED EXCEPTION]");
				}
			}
			else
			{
				try
				{
					smtpClient.sendMessage(msg_toSend);
				}
				catch (Exception e)
				{
					System.out.print(e.getMessage());
					compScreen.getSendFailedGui(e.getMessage());
					break;
				}
				msg_toSend = null;
				compScreen.notifySendSuccess();
				exit = true;
			}

		}
	}
	/**
	 * Connect to the server
	 *
	 */
	public synchronized void connect()
	{
		smtpClient = new SmtpClient(conn, localhost);
		smtpClient.setDebug(true);
		try
		{
			if (Datas.smtpAuth)
				smtpClient.open(Datas.smtpHost, 25, Datas.ssl, Datas.pop3User, Datas.password);
			else
				smtpClient.open(Datas.smtpHost);
		}
		catch (Exception e)
		{
			System.out.print(e.getMessage());
			compScreen.getSendFailedGui(e.getMessage());
		}

	}
	/**
	 * Close connection
	 *
	 */
	public synchronized void close()
	{
		try {
			smtpClient.close();
			}catch (Exception e){ 
				System.out.print(e.getMessage());}
		exit = true;
		notify(); // wakes up the thread.
	}
	/**
	 * Send mail
	 * @param msg
	 */
	public synchronized void sendMessage(Message msg)
	{

		msg_toSend = msg;
		notify(); 
	
	
	}

	
}